"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var application_1 = require("@jupyterlab/application");
var apputils_1 = require("@jupyterlab/apputils");
var coreutils_1 = require("@jupyterlab/coreutils");
var launcher_1 = require("@jupyterlab/launcher");
var mainmenu_1 = require("@jupyterlab/mainmenu");
var grid_1 = require("./grid");
var viewer_1 = require("./viewer");
/**
 * The name of the factory that creates pdf widgets.
 */
var FACTORY = 'MapDVega';
/**
 * Command IDs for the extension.
 */
var CommandIDs;
(function (CommandIDs) {
    CommandIDs.newGrid = 'mapd:new-grid';
})(CommandIDs || (CommandIDs = {}));
/**
 * The MIME type for Vega.
 *
 * #### Notes
 * The version of this follows the major version of Vega.
 */
exports.VEGA_MIME_TYPE = 'application/vnd.vega.v3+json';
exports.EXTENSIONS = ['.vega', '.mapd.vega', '.mapd.vg.json',
    '.mapd.vega.json', '.vg.json', '.vega.json'];
var PLUGIN_ID = 'jupyterlab-mapd:plugin';
/**
 * The MapD-Vega file type.
 */
var mapdFileType = {
    name: 'mapd-vega',
    displayName: 'MapD Vega',
    fileFormat: 'text',
    extensions: exports.EXTENSIONS,
    mimeTypes: [exports.VEGA_MIME_TYPE],
    iconClass: 'jpMaterialIcon jp-VegaIcon'
};
/**
 * The pdf file handler extension.
 */
var mapdPlugin = {
    activate: activateMapDViewer,
    id: PLUGIN_ID,
    requires: [launcher_1.ILauncher, application_1.ILayoutRestorer, mainmenu_1.IMainMenu, coreutils_1.ISettingRegistry],
    autoStart: true
};
function activateMapDViewer(app, launcher, restorer, mainMenu, settingRegistry) {
    var viewerNamespace = 'mapd-viewer-widget';
    var gridNamespace = 'mapd-grid-widget';
    var factory = new viewer_1.MapDViewerFactory({
        name: FACTORY,
        modelName: 'text',
        fileTypes: ['json', 'mapd-vega', 'vega3', 'vega3'],
        defaultFor: ['mapd-vega'],
        readOnly: true
    });
    var viewerTracker = new apputils_1.InstanceTracker({
        namespace: viewerNamespace
    });
    // Handle state restoration.
    restorer.restore(viewerTracker, {
        command: 'docmanager:open',
        args: function (widget) { return ({ path: widget.context.path, factory: FACTORY }); },
        name: function (widget) { return widget.context.path; }
    });
    app.docRegistry.addFileType(mapdFileType);
    app.docRegistry.addWidgetFactory(factory);
    factory.widgetCreated.connect(function (sender, widget) {
        // Notify the instance tracker if restore data needs to update.
        widget.context.pathChanged.connect(function () { viewerTracker.save(widget); });
        viewerTracker.add(widget);
        var types = app.docRegistry.getFileTypesForPath(widget.context.path);
        if (types.length > 0) {
            widget.title.iconClass = types[0].iconClass;
            widget.title.iconLabel = types[0].iconLabel;
        }
    });
    var gridTracker = new apputils_1.InstanceTracker({
        namespace: gridNamespace
    });
    // Handle state restoration.
    restorer.restore(gridTracker, {
        command: CommandIDs.newGrid,
        args: function () { return null; },
        name: function (widget) { return widget.id; }
    });
    app.commands.addCommand(CommandIDs.newGrid, {
        label: 'MapD Explorer',
        execute: function () {
            var grid = new grid_1.MapDGrid(factory.defaultConnection);
            grid.id = "mapd-grid-widget-" + Private.id++;
            grid.title.label = "MapD Explorer " + Private.id;
            grid.title.closable = true;
            grid.title.iconClass = 'mapd-MapD-logo';
            gridTracker.add(grid);
            app.shell.addToMainArea(grid);
            app.shell.activateById(grid.id);
            return grid;
        }
    });
    mainMenu.fileMenu.newMenu.addGroup([{ command: 'mapd:new-grid' }], 50);
    launcher.add({
        displayName: 'MapD Explorer',
        category: 'Other',
        rank: 0,
        iconClass: 'mapd-MapD-logo',
        callback: function () { return app.commands.execute(CommandIDs.newGrid); }
    });
    // Update the default connection data for viewers that don't already
    // have it defined.
    var onSettingsUpdated = function (settings) {
        var defaultConnection = settings.get('defaultConnection').composite;
        if (!defaultConnection) {
            return;
        }
        factory.defaultConnection = defaultConnection;
        viewerTracker.forEach(function (viewer) {
            if (!viewer.connection) {
                viewer.connection = defaultConnection;
            }
        });
        gridTracker.forEach(function (grid) {
            if (!grid.connection) {
                grid.connection = defaultConnection;
            }
        });
    };
    // Fetch the initial state of the settings.
    Promise.all([settingRegistry.load(PLUGIN_ID), app.restored])
        .then(function (_a) {
        var settings = _a[0];
        settings.changed.connect(onSettingsUpdated);
        onSettingsUpdated(settings);
    }).catch(function (reason) {
        console.error(reason.message);
    });
}
/**
 * Export the plugin as default.
 */
var plugin = mapdPlugin;
exports.default = plugin;
/**
 * A namespace for private statics.
 */
var Private;
(function (Private) {
    Private.id = 0;
})(Private || (Private = {}));
